@define-color color1 #ED6500;
@define-color color2 #B10080;
@define-color color3 #009384;
@define-color color4 #A7E100;
@define-color bg #13181C;
@define-color active #324A5E;
@define-color echo #007D70;
@define-color error #981E1F;
@define-color warning #F9A825;
@define-color warning-text #434343;
@define-color text white;
@define-color text-inv black;
@define-color lines #cccccc;
@define-color switch-scale-bg #3584E4;

* {
    color: @text;
}

junction,
list row,
treeview.view,
combobox arrow,
window {
    background-color: @bg;
}

button {
    background-color: @bg;
}

button:focus,
button:active {
    background-color: @active;
}

.button_active {
    background-color: @active;
    border-radius: 1em;
}

button.color1 {
    border-bottom-color: @color1;
}

button.color2 {
    border-bottom-color: @color2;
}

button.color3 {
    border-bottom-color: @color3;
}

button.color4 {
    border-bottom-color: @color4;
}

button.active {
    background-color: @active;
}

.active image,
button.update {
    background: @color3;
    border-color: @color3;
}

button.invalid {
    background: @error;
    border-color: @error;
}

combobox box button {
    border-color: @lines;
}

treeview.view check,
switch {
    background-color: @active;
}

treeview.view check:checked,
switch:checked {
    background-color: @switch-scale-bg;
}

treeview.view check:checked {
    -gtk-icon-source: url("./styles/z-bolt/images/complete.svg");
}

entry {
    background-color: @active;
    border-color: @bg;
}

label {
    color: @text;
}

menu {
    background-color: @bg;
    border-color: @lines;
}

menuitem {
    background-color: @bg;
    border-color: @lines;
}

trough {
    background-color: @active;
    color: @text;
    border-color: @text-inv;
}

trough highlight,
trough progress {
    background-color: @switch-scale-bg;
    color: @text;
    border-color: @text-inv;
}

scale-mark {
    color: @text;
}

scrollbar,
scrollbar button,
scrollbar trough {
    background-color: @bg;
}

scrollbar slider {
    background-color: @active;
}

textview .time {
    color: @lines;
}

.dialog {
    border: @bg;
    background-color: @bg;
}

.horizontal_togglebuttons_active {
    background-color: @active;
    border-color: @color1;
    box-shadow: 0 0 0 .2em @lines inset;
}

.horizontal_togglebuttons {
    border-color: @lines;
}

.fan_slider {
    color: @text;
}

.message_popup {
    border-bottom-color: @lines;
}

.numpad_key {
    border-color: @lines;
}

.message {
    border-color: @error;
    color: @text;
    background-color: @error;
}

.message button {
    background-color: @text;
    color: @text-inv;
}

.message_popup_echo {
    background-color: @echo;
}

.message_popup_warning {
    background-color: @warning;
}

.message_popup_warning label {
    color: @warning-text;
}
.message_popup_error {
    background-color: @error;
}

.keyboard_pad {
    background-color: @bg;
}

.dialog-error {
    border-bottom-color: @error;
}

.dialog-warning {
    border-bottom-color: @warning;
}

.dialog-info {
    border-bottom-color: @color4;
}

.dialog-default {
    border-bottom-color: @color3;
}

.dialog-secondary {
    border-bottom-color: @color2;
}

.dialog-primary {
    border-bottom-color: @color1;
}
