/* Colorized theme created by Alfredo Monclus 2021 */
/* Using solarized colors */
@define-color solarized-base0 #839496;
@define-color solarized-base1 #93a1a1;
@define-color solarized-base2 #eee8d5;
@define-color solarized-base3 #fdf6e3;
@define-color solarized-base00 #657b83;
@define-color solarized-base01  #586e75;
@define-color solarized-base02 #073642;
@define-color solarized-base03 #002b36;
@define-color solarized-yellow #b58900;
@define-color solarized-orange #cb4b16;
@define-color solarized-red #dc322f;
@define-color solarized-magenta #d33682;
@define-color solarized-violet #6c71c4;
@define-color solarized-blue #268bd2;
@define-color solarized-cyan #2aa198;
@define-color solarized-green #859900;

* {
    color: @solarized-base3;
}

junction,
list row,
treeview.view,
window {
    background-color: @solarized-base03;
}

button {
    background-color: @solarized-base03;
}

.active image,
button:active {
    background-color: @solarized-orange;
}

.button_active {
    border-color: @solarized-base02;
    border-style: solid;
    border-radius: 1em;
    background-color: @solarized-base02;
}

button.color1 {
    background-color: @solarized-base02;
    border-bottom-color: @solarized-orange;
    border-radius: 1em;
}

button.color2 {
    background-color: @solarized-base02;
    border-bottom-color: @solarized-magenta;
    border-radius: 1em;
}

button.color3 {
    background-color: @solarized-base02;
    border-bottom-color: @solarized-cyan;
    border-radius: 1em;
}

button.color4 {
    background-color: @solarized-base02;
    border-bottom-color: @solarized-green;
    border-radius: 1em;
}

button:focus,
button.color1:focus,
button.color2:focus,
button.color3:focus,
button.color4:focus,
button.update:focus,
button.invalid:focus,
.dialog button:focus
{
    outline: .1em dashed @solarized-orange;
    outline-offset: .1em;
}

button.color1:active,
button.color2:active,
button.color3:active,
button.color4:active,
button.update:active,
button.invalid:active,
.dialog button:active {
    border-style: none;
    margin-top: .6em;
}

button.active {
    background-color: @solarized-orange;
}

button.update {
    background-color: @solarized-base02;
    border: 0;
    border-bottom: .3em solid;
    border-bottom-color: @solarized-green;
    border-radius: .75em;
}

button.invalid {
    background-color: @solarized-base02;
    border: 0;
    border-bottom: .3em solid;
    border-bottom-color: @solarized-red;
    border-radius: .75em;
}

menu,
menuitem,
combobox button,
combobox arrow,
combobox menuitem,
popover,
popover button {
    background-color: @solarized-base02;
    border-color: @solarized-base03;
}

switch {
    background-color: @solarized-base02;
}
switch:checked {
    background-color: @solarized-green;
}

treeview.view check {
    background-image: none;
    background-color: @solarized-base02;
    border-radius: .51em;
}

treeview.view check:checked {
    background-image: none;
    background-color: @solarized-base02;
    -gtk-icon-source: url("./styles/colorized/images/complete.svg");
}

entry {
    background-color: @solarized-base02;
    border-color: @solarized-base03;
}

label {
    color: @solarized-base3;
}

trough {
    background-color: @solarized-base02;
    color: @solarized-base3;
    border-color: @solarized-base03;
    border-radius: 1em;
}

trough highlight,
trough progress {
    background-color:  @solarized-violet;
    color: @solarized-base3;
    border-color: @solarized-base03;
    border-top-left-radius: 1em;
    border-bottom-left-radius: 1em;
}

scale-mark {
    color: @solarized-base3;
}

scrollbar,
scrollbar button,
scrollbar trough {
    background-color: @solarized-base03;
}

scrollbar slider {
    background-color: @solarized-base02;
}

textview .time {
    color:  @solarized-base1;
}

.dialog {
    border: @solarized-base03;
    background-color: @solarized-base03;
}

.dialog button {
    background-color: @solarized-base02;
    border-radius: .75em;
}

.horizontal_togglebuttons  {
    border-color: @solarized-base03;
    background-color: @solarized-base02;
}

.horizontal_togglebuttons_active {
    background-color: @solarized-orange;
}

.fan_slider {
    color: @solarized-base3;
}

.message_popup {
    border-bottom-color: @solarized-base3;
}

.message_popup button {
    background-color:  @solarized-red;
}

.message_popup_echo,
.message_popup_echo button {
    background-color: @solarized-green;
}

.message_popup_warning,
.message_popup_warning button {
    background-color:  @solarized-orange;
}

.message_popup_error,
.message_popup_error button {
    background-color:  @solarized-red;
}

.numpad_key {
    border-color: @solarized-base03;
    background-color: @solarized-base02;
}

.printing-status {
    color: @solarized-base3;
}

.printing-progress-text {
    color: @solarized-base3;
}

.printing-status-label {
    color:  @solarized-base1;
}

.message {
    border-color: @solarized-red;
    color: @solarized-base3;
    background-color: @solarized-red;
}

.message button {
    background-color: @solarized-base3;
    color: @solarized-base03;
}

.warning {
    background-color:  @solarized-yellow;
}

.error {
    background-color: @solarized-red;
}

.keyboard_pad {
    background-color: @solarized-base02;
}

.dialog-error {
    border-bottom-color: @solarized-red;
}

.dialog-warning {
    border-bottom-color: @solarized-orange;
}

.dialog-info {
    border-bottom-color: @solarized-cyan;
}

.dialog-default {
    border-bottom-color: @solarized-blue;
}

.dialog-secondary {
     border-bottom-color: @solarized-magenta;
}

.dialog-primary {
     border-bottom-color: @solarized-green;
}
