import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
from ks_includes.screen_panel import ScreenPanel


class Panel(ScreenPanel):
    def __init__(self, screen, title):
        title = title or _("Example")
        super().__init__(screen, title)

        # Create gtk items here

        self.content.add(Gtk.Box())
