/* Material-light theme created by Alfredo Monclus 2021 */
@define-color bg #FAFAFA;
@define-color text black;
@define-color color1 #FFF59D;
@define-color color2 #F48FB1;
@define-color color3 #C5E1A5;
@define-color color4 #80DEEA;
@define-color border #BDBDBD;
@define-color active #FFAB91;
@define-color warning #FFE082;
@define-color error #EF9A9A;
@define-color echo #a1ef9a;
@define-color textview #212121;

* {
    color: @text ;
}

junction,
list row,
treeview.view,
window {
    background-color: @bg;
}

button {
    background-color: @bg;
}

button label{
    color: @text;
}

.active image,
button:active {
    background-color: @active;
}

.button_active {
    border-color: @border;
    border-style: solid;
    border-width: .25em;
    background-color: @bg;
    box-shadow: .1em .1em @border;
    border-radius: 1em;
}

button.color1 label,
button.color2 label,
button.color3 label,
button.color4 label {
    color: @text;
}

button.color1 {
    background-color: @color1;
    border-color: @color1;
    box-shadow: .1em .1em @border;
    border-radius: 1em;
}

button.color2 {
    background-color: @color2;
    border-color: @color2;
    box-shadow: .1em .1em @border;
    border-radius: 1em;
}

button.color3 {
    background-color: @color3;
    border-color: @color3;
    box-shadow: .1em .1em @border;
    border-radius: 1em;
}

button.color4 {
    background-color: @color4;
    border-color: @color4;
    box-shadow: .1em .1em @border;
    border-radius: 1em;
}

button:focus,
button.color1:focus,
button.color2:focus,
button.color3:focus,
button.color4:focus,
button.update:focus,
button.invalid:focus,
.dialog button:focus
{
    outline: .1em dashed @active;
    outline-offset: .1em;
}

button.color1:active,
button.color2:active,
button.color3:active,
button.color4:active,
button.update:active,
button.invalid:active,
.dialog button:active {
    border-style: none;
    margin-top: 0.6em;
    box-shadow: 0 0;
}

button.active {
    background-color: @active;
}

button.update {
    background-color: @color3;
    border-color: @color3;
}

button.invalid {
    background-color: @error;
    border-color: @error;
}

combobox window.popup {
    background-color: @bg;
}

combobox box button {
    background-color: @bg;
    border-color: @border;
}

treeview.view check,
switch {
    background-color: @border;
}

treeview.view check:checked,
switch:checked {
    background-color: @color3;
}

treeview.view check:checked {
    -gtk-icon-source: url("./styles/material-light/images/complete.svg");
}

treeview.view check,
switch slider,
scale trough,
progressbar trough,
scale trough slider {
    border: .1em solid @border;
}

entry {
    border-color: @border;
    background-color: @bg;
}

label {
    color: @text;
}

menu {
    background-color: @bg;
    border-color: @border;
}

menuitem {
    background-color: @bg;
    border-color: @border;
}

trough {
    background-color: transparent;
    color: @border;
    border-radius: 1em;
}

trough highlight,
trough progress {
    background-color: @color2;
    color: @bg;
    border-color: @border;
    border-top-left-radius: 1em;
    border-bottom-left-radius: 1em;
}


progressbar trough progress {
    border-top-right-radius: 1em;
    border-bottom-right-radius: 1em;
}

scale-mark {
    color: @bg;
}

scrollbar,
scrollbar button,
scrollbar trough {
    background-color: transparent;
}

scrollbar slider {
    background-color: @border;
}

textview {
    background-color: @textview;
}

textview .time {
    color: @border;
}

.horizontal_togglebuttons  {
    border-color: @border;
    background-color: @bg;
}

.horizontal_togglebuttons_active {
    background-color: @active;
}

.horizontal_togglebuttons_active label {
    color: @text;
}

.fan_slider {
    color: @bg;
}

.frame-item {
    border-bottom-color: @border;
}

.message_popup {
    border-bottom-color: @border;
}

.message_popup_echo,
.message_popup_echo button {
    background-color: @echo;
}

.message_popup_warning,
.message_popup_warning button {
    background-color: @warning;
}

.message_popup_error,
.message_popup_error button {
    background-color: @error;
}

.numpad_key {
    border-color: @border;
    background-color: @bg;
}

.printing-status {
    color: @text;
}

.printing-progress-text {
    color: @text;
}

.printing-status-label {
    color: @text;
}

.message {
    border-color: @error;
    color: @bg;
    background-color: @error;
}

.message button {
    background-color: @bg;
    color: @text;
}

.warning {
    background-color: @warning;
}

.error {
    background-color: @error;
}

.keyboard_pad {
    background-color: @border;
}

.dialog {
    border: @border;
    background-color: @bg;
}

.dialog button {
    border-radius: .75em;
}

.dialog-error {
    background-color: @error;
    border-bottom-color: @error;
}

.dialog-warning {
    background-color: @warning;
    border-bottom-color: @warning;
}

.dialog-info {
    background-color: @color4;
    border-bottom-color: @color4;
}

.dialog-default {
    background-color: @color3;
    border-bottom-color: @color3;
}

.dialog-secondary {
    background-color: @color2;
    border-bottom-color: @color2;
}

.dialog-primary {
    background-color: @color1;
    border-bottom-color: @color1;
}
