img {
  display: block;
  margin: 0 auto;
}

.md-typeset .md-typeset__table {
  display: block;
}
.md-typeset__table table:not([class]) {
  display: table;
}

h2, h3, h4 {
  padding: 0.5em 0;
  text-align: center;
  background-color: rgba(0, 0, 0, 0.5);
}

.md-typeset table:not([class]) td,
.md-typeset table:not([class]) th {
  vertical-align: middle;
}

.md-typeset table img {
  margin: 0;
  display: inline-block;
}
