/* Material-darker theme created by Alfredo Monclus 2021 */
@define-color color_1 #e2e2e2;
@define-color color_2 #2196f3;
@define-color color_3 #fb8c00;
@define-color color_4 #9E9E9E;
@define-color color_5 #121212;
@define-color background-color_2 #1c1c1c;
@define-color background-color_3 #424242;
@define-color background-color_4 #252525;
@define-color background-color_5 #2f5631;
@define-color background-color_6 #006064;
@define-color background-color_7 #545454;
@define-color background-color_8 #006412;
@define-color background-color_9 #EF6C00;
@define-color background-color_10 #B71C1C;
@define-color background-color_12 #f9b274;
@define-color background-color_13 #CF6679;
@define-color background-color_14 #090909;
@define-color border-color_6 #a2a2a2;
@define-color border-color_7 #282828;

* {
    color: #e2e2e2;
    font-family: RobotoMedium;
}

junction,
list row,
treeview.view,
window {
    background-color: #121212;
}

button {
    background-color: #121212;
}

button:active {
    background-color: #424242;
}

.button_active {
    border-color: #424242;
    border-style: solid;
    background-color: #424242;
}

button.color1,
button.color2,
button.color3,
button.color4 {
    background-color: #252525;
    border-color: #252525;
    border-radius: .25em;
    padding-top: 0.5em;
    padding-bottom: 0.5em;
}


button:focus,
button.color1:focus,
button.color2:focus,
button.color3:focus,
button.color4:focus,
button.update:focus,
button.invalid:focus,
.dialog button:focus
{
    background-color: #424242;
    border-color: #424242;
}

button.color1:active,
button.color2:active,
button.color3:active,
button.color4:active,
button.update:active,
button.invalid:active,
.dialog button:active {
    border-style: none;
    background-color: #424242;
    margin-top: 0.55em;
}

.active image,
button.update {
    color: #2196f3;
    background-color: #121212;
    border-color: #2196f3;
    border-radius: .5em;
    border-width: .1em;
}

button.invalid {
    color: #fb8c00;
    background-color: #121212;
    border-color: #fb8c00;
    border-radius: .5em;
    border-width: .1em;
}

menu,
menuitem,
combobox button,
combobox arrow,
combobox menuitem {
    background-color: #252525;
    border-color: #121212;
}

treeview.view check,
switch {
    background-color: #1c1c1c;
}

treeview.view check:checked,
switch:checked {
    background-color: #2f5631;
}

treeview.view check:checked {
    -gtk-icon-source: url("./styles/material-darker/images/complete.svg");
}

label {
    color: #e2e2e2;
}

trough {
    background-color: #121212;
    color: #e2e2e2;
}

trough highlight,
trough progress {
    background-color: #2f5631;
    color: #e2e2e2;
}

scale-mark {
    color: #e2e2e2;
}

scrollbar,
scrollbar button,
scrollbar trough {
    background-color: #121212;
}

scrollbar slider {
    background-color: #1c1c1c;
}

textview .time {
    color: #9E9E9E;
}

.active_device {
    background-color: #1c1c1c;
}

.dialog {
    border: 0;
    background-color: #121212;
}

.dialog button {
    background-color: #252525;
    border-radius: .25em;
}


.dialog-error {
    border-bottom-color: #CF6679;
}

.dialog-warning {
    border-bottom-color: #f9b274;
}

.dialog-info {
    border-bottom-color: @color_4;
}

.dialog-default {
    border-bottom-color: @color_3;
}

.dialog-secondary {
     border-bottom-color: @color_2;
}

.dialog-primary {
     border-bottom-color: @color_1;
}

.horizontal_togglebuttons  {
    border-color: #545454;
    border-width: .1em;
    background-color: #252525;
}

.horizontal_togglebuttons_active {
    background-color: #545454;
}

.fan_slider {
    color: #e2e2e2;
}

.message_popup {
    border-bottom-color: #e2e2e2;
}

.message_popup_echo,
.message_popup_echo button {
    background-color: #006412;
}

.message_popup_warning,
.message_popup_warning button {
    background-color: #EF6C00;
}

.message_popup_error,
.message_popup_error button {
    background-color: #B71C1C;
}

.numpad_key {
    border-color: #545454;
    background-color: #252525;
}

.printing-status {
    color: #e2e2e2;
}

.printing-progress-text {
    color: #e2e2e2;
}

.printing-status-label {
    color: #9E9E9E;
}

.message {
    border-color: #B71C1C;
    color: #e2e2e2;
    background-color: #B71C1C;
}

.message button {
    background-color: #e2e2e2;
    color: #121212;
}

.keyboard_pad {
    background-color: #090909;
}
